// * This code is licensed under:
// * JRunMC License, Version 1.0
// * - for license details see http://runmc.sourceforge.net/License.txt
// *
// * Copyright (c) 2005 by S.Chekanov (chekanov@mail.desy.de). 
// * All rights reserved.
package JRunMC.ui;


import JRunMC.util.*;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeSet;
import java.lang.*;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;


/**
 *  Static singleton class organizing actions, mainly depending on the command 
 *  menus, and action related assisting routines. This class manages the system's
 *  idle time counter. 
 * <p>
 */
public class ActionHandler
{
 

//   public static final long serialVersionUID = 2;
   private static ActionListener actionListener = new MainActionListener();
   private static DocumentListener docListener = new DocumentChangeListener();
   private static long lastUseTime;
   private static Action clearClipAction;
   
   /**
    *  Singleton class. 
    */
   private ActionHandler ()
   {
   }

   public static void init (  )
   {
      lastUseTime = System.currentTimeMillis();
      
      // define Action "menu.edit.clearclip"
      clearClipAction = new AbstractAction( 
            ResourceLoader.getCommand( "menu.edit.clearclip" ),
            ResourceLoader.getImageIcon( "clearclip" ) )
      {

         public static final long serialVersionUID = 102112; 
         public void actionPerformed( ActionEvent arg )
         {
            actionListener.actionPerformed( arg );
         }
      };
      clearClipAction.putValue( Action.SHORT_DESCRIPTION, 
            ResourceLoader.getCommand( "toolbar.clearclip.tooltip" ) );
      clearClipAction.putValue( Action.ACTION_COMMAND_KEY, "menu.edit.clearclip" );
      
   }  // init
   
   /** Returns the "ClearClipboard" action which is defined in this class. */
   public static Action getClearClipAction ()
   {
      return clearClipAction;
   }
   
   /** Returns the <code>ActionListener</code> which is defined in this class. */
   public static ActionListener getMainActionListener ()
   {
      return  actionListener;
   }

   /** Returns the actual idle time of the application. */ 
   public static long getIdleTime ()
   {
      return System.currentTimeMillis() - lastUseTime;
   }

   /** Resets the application's idle counter to zero (current time). */
   public static void resetIdleTime ()
   {
      lastUseTime = System.currentTimeMillis();
   }
   
   /** Causes a new password file to be created and displayed in the GUI.
    *  (Includes user passphrase input.)
    */



/**
 *  An instance of this class listenes to <code>JTextComponent</code>s and 
 *  resets the system's idle-time counter whenever a modification occurs on
 *  a component's text.      
 */
private static class DocumentChangeListener implements DocumentListener
{

public void changedUpdate ( DocumentEvent e )
{}

public void insertUpdate ( DocumentEvent e )
{
   resetIdleTime();
}

public void removeUpdate ( DocumentEvent e )
{
   resetIdleTime();
}
} // inner class DocumentChangeListener
 
/**
 *  This class is the main listener to action commands. Those are majorly
 *  coming from the GUI's menubars and toolbars.
 *  
 */
private static class MainActionListener implements ActionListener
{

public void actionPerformed ( ActionEvent arg0 )
{

   String cmd, hstr;
   boolean isSelected;
   
   lastUseTime = System.currentTimeMillis();
   cmd = arg0.getActionCommand();

//   System.out.println("Debug: action:"+cmd);  
 
   if ( cmd.equals( "" ) )
      return;
  
 
   else if ( cmd.equals( "menu.help.mc" ) )
   {

    try {  
    Global.mainFrame.ShowMess("Starting Acroread to open Monte Carlo manual.. wait");  
    Process p = Runtime.getRuntime().exec(
    SetEnv.RUNDIR+"/main/scripts/show_manual_mc.sh "+Integer.toString(SetIni.MCtype)+" &");
    } catch (Exception err) {  err.printStackTrace(); }

   }



  else if ( cmd.equals( "menu.help.runmc" ) )
   {

     Global.mainFrame.ShowMess("Show RunMC help");
     new HelpRunMC();

/*
    try {
     Global.mainFrame.ShowMess("Starting Acroread to open RunMC manual.. wait");
     Process p = Runtime.getRuntime().exec(SetEnv.RUNDIR+"/main/scripts/show_manual.sh &");
        } catch (Exception err) {  err.printStackTrace(); }
*/

 }
 else if ( cmd.equals( "menu.help.paper" ) )
   {

       Global.mainFrame.ShowMess("Show RunMC paper");
       try {
        Global.mainFrame.ShowMess("Starting Acroread to open the RunMC paper.. wait");
        Process p = Runtime.getRuntime().exec(
        SetEnv.RUNDIR+"/main/scripts/show_paper.sh &");
        } catch (Exception err) {  err.printStackTrace();
        } 

   }

else if ( cmd.equals( "menu.file.exit" ) )
   {

    String ss="Before exit, save the current settings to \""+SetIni.MOutput+".mc\" file? \n";
    ss=ss+"Old file \""+SetIni.MOutputOld+".mc\" will be removed!"; 
    String sn=SetIni.MOutput+".mc";

    JOptionPane dialog1 = new JOptionPane();
    int ask=dialog1.showConfirmDialog(dialog1, ss,
    "Question", JOptionPane.YES_NO_OPTION,
    JOptionPane.QUESTION_MESSAGE, null );


     if (ask == JOptionPane.YES_OPTION) { 
     try {
            // remove old MC file
            removeOldMC();
            // make new MC file
            SetIni.FileWriteMC(sn);
        } catch (Exception e) {
           Global.mainFrame.ShowMess(SetIni.MOutput+".mc  was not saved!");

        }
       }



// start exiting 
          String sh = "bash"; String cop = "-c";
          String cmd1 = "pkill histoview";
          String[] exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
         //       System.out.println("histoview killed");
          } catch (Exception err) {err.printStackTrace(); }


          cmd1 = "pkill ntupview";
          exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
          //      System.out.println("ntupview  killed");
          } catch (Exception err) {err.printStackTrace(); }


// save options
      if (!Options.checkOption()) {

      JOptionPane dialog2 = new JOptionPane();
      String s="Some options are different from the defaut values. Save them for future?";
      String mesg = "<html><b>"+s+"</b></html>";

       int ask2=dialog2.showConfirmDialog(dialog2, mesg, "Question",
              JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE, null );
       if (ask2 == JOptionPane.YES_OPTION) Options.save();
       if (ask2 == JOptionPane.NO_OPTION)  Options.reset();
       }

// exit
     System.exit(0);

 }
   else if ( cmd.equals( "menu.help.about" ) )
   {
        Global.mainFrame.ShowMess("About dialog");
        new AboutDialog();
 
   }


  else if ( cmd.equals( "menu.help.selfupdate" ) )
   {
        Global.mainFrame.ShowMess("Self update");

      JOptionPane dialog3 = new JOptionPane();
      String s="This will update the current GUI to the most recent version.<br> Do you want to start the update?";
      String mesg = "<html><b>"+s+"</b></html>";
       int ask2=dialog3.showConfirmDialog(dialog3, mesg, "Question",
              JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE, null );
       if (ask2 == JOptionPane.YES_OPTION) GUIwebLoader.init(); 
//       if (ask2 == JOptionPane.NO_OPTION)  Options.save();  

   }


  else if ( cmd.equals( "menu.help.system" ) )
   {
        Global.mainFrame.ShowMess("System information");
        new SystemInfo();

//          new SystemDialog();
   }


 else if ( cmd.equals( "menu.file.openmc" ) )
   {
        Global.mainFrame.ShowMess("Open MC file");
        Global.mainFrame.OpenMCfile();

//          new SystemDialog();
   }


else if ( cmd.equals( "menu.project.open" ) )
   {
        Global.mainFrame.ShowMess("Open RMC file");
        new RMCLoader();
   }


 else if ( cmd.equals( "menu.project.browser" ) )
   {
          String sh = "bash"; String cop = "-c";
          String cmd1 = "$RUNMC/main/jrunmc/jhep/ehep $RUNMC/proj/user-init.cpp";
          String[] exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
               Global.mainFrame.ShowMess("Start project browser.."); 
          } catch (Exception err) {err.printStackTrace(); }
          //System.out.println("start browser");

     }  

else if ( cmd.equals( "menu.project.save" ) )
   {


// get new MC name 
     String sn=SetIni.MOutput+".mc";
// make new MC file 
     try {
            // remove old MC file
            removeOldMC();
            // make new MC file
            SetIni.FileWriteMC(sn);
        } catch (Exception e) {
           Global.mainFrame.ShowMess(SetIni.MOutput+".mc  was not saved!");

        }


// naw get a proper name (i.e. from the existing file)  
    String name=RMCaction.getName();

    if (name.equals("none")) {
       String rError="cannot find <proj>.rmc file!";
       JOptionPane dialogError = new JOptionPane();
       dialogError.showMessageDialog(dialogError,rError,"Error",JOptionPane.ERROR_MESSAGE);

    } else { 
 

    JOptionPane dialogsaveRMC = new JOptionPane();
    int asksaveRMC=dialogsaveRMC.showConfirmDialog(dialogsaveRMC,
     "Save current project to \""+name+".rmc\" file?",
     "Question", JOptionPane.YES_NO_OPTION,
       JOptionPane.QUESTION_MESSAGE, null );
       if (asksaveRMC == JOptionPane.YES_OPTION) {
        try {
            // make new lists
             SetRMC.Read();
             SetRMC.LoadAll(); 

         //  Global.mainFrame.ShowMess(sn+" file was saved");
        } catch (Exception e) {
           Global.mainFrame.ShowMess(SetIni.MOutput+".mc  was not saved!");

        }
         // check file

         String path=SetEnv.RUNDIR;
         if (SetEnv.OSsys.equals("windows") ) path=SetEnv.RUNDIRwin32;

         String ss=path+File.separator+"archive"+File.separator+SetIni.MOutput+".rmc";
 
         File oldrmc= new File(  ss  );

/*
          String sh = "bash"; String cop = "-c";
          String cmd1 = SetEnv.RUNDIR+"/main/scripts/proj_create.sh &";
          String[] exe1 = new String[] {sh, cop, cmd1};
*/  

         if (oldrmc.exists()) {
         JOptionPane dialog3 = new JOptionPane();
         int ask3=dialog3.showConfirmDialog(dialog3,
         "RMC file with this name exists, overwrite?",
         "Question", JOptionPane.YES_NO_OPTION,
         JOptionPane.QUESTION_MESSAGE, null );
         if (ask3 == JOptionPane.YES_OPTION) {

//         try {

            RMCaction.save();
 
//           Process p = Runtime.getRuntime().exec( exe1 );
           Global.mainFrame.ShowMess(SetIni.MOutput+".rmc  project was saved");
          
           SetRMC.Read(); SetRMC.LoadAll(); // load infor from all RMC

 
//         } catch (Exception err) {  err.printStackTrace();}
         } else if (ask3 == JOptionPane.NO_OPTION) {
            Global.mainFrame.ShowMess(SetIni.MOutput+".rmc  was canceled"); 
         }
         } else { // if file does not exits

 //         try {Process p = Runtime.getRuntime().exec( exe1 );
 //        } catch (Exception err) {  err.printStackTrace();}

          RMCaction.save();
       
          SetRMC.Read(); SetRMC.LoadAll(); // load info from all RMC
          Global.mainFrame.ShowMess( SetIni.MOutput+".rmc  project was saved" );
         }
            } else if (asksaveRMC == JOptionPane.NO_OPTION) {
        } else { System.out.print("Something is wrong"); };


    } // else correct file


   }


else if ( cmd.equals( "menu.tools.control.card" ) )
   {


     File f1= new File(SetIni.card[SetIni.MCtype]);
     SetIni.ShowStatus="Last action: Start steering card editor";
          if (!f1.exists()) {
            JOptionPane dialog1 = new JOptionPane();
            int ask=dialog1.showConfirmDialog(dialog1,
             "Create new "+SetIni.card[SetIni.MCtype]+" ?",
             "Question", JOptionPane.YES_NO_OPTION,
       JOptionPane.QUESTION_MESSAGE, null );
       if (ask == JOptionPane.YES_OPTION) {
       EditorCards s1 = new EditorCards(SetIni.card[SetIni.MCtype],"new"); s1.setVisible(true);
       };
       } else { EditorCards s1 = new EditorCards(SetIni.card[SetIni.MCtype],"old");
       s1.setVisible(true); 
       } 
       

}


else if ( cmd.equals( "menu.tools.histogram.editor" ) )
   {

//    EditorHisto histo= new EditorHisto();
    Global.mainFrame.histo.setVisible(true);
}



else if ( cmd.equals( "menu.file.savemc" ) )
   {

      JOptionPane dialog1 = new JOptionPane();
      String ss="Save current GUI settings to \""+SetIni.MOutput+".mc\" file?\n";
      ss=ss+"Old file \""+SetIni.MOutputOld+".mc\" will be removed!"; 

      int ask=dialog1.showConfirmDialog(dialog1, ss,
       "Question", JOptionPane.YES_NO_OPTION,
       JOptionPane.QUESTION_MESSAGE, null );
       if (ask == JOptionPane.YES_OPTION) {

                 // remove old MC file 
                 removeOldMC();

                 // write file
                 String sn=SetIni.MOutput+".mc";
                 // save histograms first
                 try {
                   SetIni.FileWriteMC(sn);
               } catch (Exception e) {
                  System.out.println("ActionHandler: Error in file openning");
                  // show error
                  JOptionPane dialog2 = new JOptionPane();
                  dialog2.showMessageDialog(dialog2, "Cannot write "+sn,
                  "Error", JOptionPane.ERROR_MESSAGE );  };
                     Global.mainFrame.ShowMess( sn+" file was saved"); 
                    } else if (ask == JOptionPane.NO_OPTION) {
                    } else {
                      System.out.print("Something is wrong");
                    }

}

else if ( cmd.equals( "menu.project.info" ) )
   {

// start RMC viewer
        try {
         Global.mainFrame.ShowMess("Start RMC viewer ..");
      //   new RMCviewer();
         EditorInfo ed= new  EditorInfo(); ed.ShowHTML();
        } catch (Exception err) {  err.printStackTrace();
        }    

}


else if ( cmd.equals( "menu.project.info.edit" ) )
   {

// start RMC viewer
        try {
         Global.mainFrame.ShowMess("Start RMC info editor ..");
         EditorInfo ed= new  EditorInfo(); ed.Show(); 
        } catch (Exception err) {  err.printStackTrace();
        }

}




  else
      System.out.println("** unrecognized command: " + cmd );
   
}  // actionPerformed


}



// remove old MC when there is somthing to remove 
private  static void removeOldMC() { 
 // get path
         String path=SetEnv.RUNDIR;
         if (SetEnv.OSsys.equals("windows") ) path=SetEnv.RUNDIRwin32;


         String oldf=path+File.separator+"proj"+File.separator+SetIni.MOutputOld+".mc";
         File myFile = new File(oldf);
// remove prevous file to exclude a dublicate
        try {  myFile.delete();
        }
        catch (SecurityException e) {
//          System.out.println("Caught security exception trying to delete file " + myFile.toString());
        }

 }


}
