/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/



//FontDialog.java - this class displays the fonts on the users system and allows the user
//to select a new font for the system. The class also displays the currently used font.
//This java file is used with permission from Beginning Java 2 by Ivor Horton (see bibliography)
//this segment ofcode used with permission
//(C)2000 Beginning Java 2 Ivor Horton
//www.wrox.com


package com.misc;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import java.awt.AWTEvent;
import java.awt.Insets;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Color;
import java.awt.GraphicsEnvironment;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JRadioButton;
import javax.swing.JSplitPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.ListSelectionModel;

import com.jpe.jpe;
import com.jpe.Constants;
import com.utils.*;

public class FontDialog extends JDialog implements ActionListener,
	ListSelectionListener, Constants
{
  // Constructor
  public FontDialog(jpe window)
  {
    // Call the base constructor to create a modal dialog
    super(window, "Font", false);

    this.window = window;

    font = window.getFont();
    fontStyle = font.getStyle();
    fontSize = font.getSize();
	  fontName = font.getName();

    // Create the dialog button panel
    JPanel buttonPane = new JPanel();

    // Create and add the buttons to the buttonPane
    buttonPane.add(ok = createButton("OK"));
    buttonPane.add(cancel = createButton("Cancel"));
    getContentPane().add(buttonPane, BorderLayout.SOUTH);

    // Code to create the data input panel...
    JPanel dataPane = new JPanel();
    dataPane.setBorder(BorderFactory.createCompoundBorder(
	  BorderFactory.createLineBorder(Color.black),
	  BorderFactory.createEmptyBorder(5, 5, 5, 5)));
    GridBagLayout gbLayout = new GridBagLayout();
    dataPane.setLayout(gbLayout);
    GridBagConstraints constraints = new GridBagConstraints();

    // Code to create the font choice and add it to the input panel
    JLabel label = new JLabel("Choose a Font");
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    gbLayout.setConstraints(label, constraints);
    dataPane.add(label);

    // Code to set up font list choice component
    GraphicsEnvironment e = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String[] fontNames = e.getAvailableFontFamilyNames();

    fontList = new JList(fontNames);
    fontList.setValueIsAdjusting(true);
    fontList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    fontList.setSelectedValue(fontName,true);
    fontList.setVisibleRowCount(4);
    fontList.addListSelectionListener(this);
    JScrollPane chooseFont = new JScrollPane(fontList);
    chooseFont.setMinimumSize(new Dimension(250,50));

    // Panel to display font sample
    JPanel display = new JPanel();
    fontDisplay = new JLabel(" a A b B c C d D 1 2 3");
    fontDisplay.setPreferredSize(new Dimension(150,20));
    display.add(fontDisplay);

    //Create a split pane with font choice at the top
    // and font display at the bottom
    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, chooseFont, display);
    gbLayout.setConstraints(splitPane, constraints);
    dataPane.add(splitPane);

    // Set up the size choice using a combobox
    JPanel sizePane = new JPanel();
    label = new JLabel("Point size");
    sizePane.add(label);
    String[] sizeList = { "8", "9", "10", "11", "12", "14", "16",
			"18", "20", "22", "24", "26"};
    chooseSize = new JComboBox(sizeList);
    chooseSize.setSelectedItem(Integer.toString(fontSize));
    chooseSize.addActionListener(this);
    sizePane.add(chooseSize);
    gbLayout.setConstraints(sizePane, constraints);
    dataPane.add(sizePane);

    // Set up style options using radio buttons
    JRadioButton bold = new JRadioButton("Bold", (fontStyle & Font.BOLD) > 0);
    JRadioButton italic = new JRadioButton("Italic", (fontStyle & Font.ITALIC) > 0);
    bold.addItemListener(new StyleListener(Font.BOLD));
    italic.addItemListener(new StyleListener(Font.ITALIC));
    JPanel stylePane = new JPanel();
    stylePane.add(bold);
    stylePane.add(italic);
    gbLayout.setConstraints(stylePane, constraints);
    //dataPane.add(stylePane);

    getContentPane().add(dataPane, BorderLayout.CENTER);
    pack();
    setVisible(false);
  }

	JButton createButton(String label)
	{
		JButton button = new JButton(label);
		button.setFont(SYSTEM_FONT);
		button.setPreferredSize(new Dimension(70,20));
		button.addActionListener(this);
		return button;
	}

	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if(source == ok)
    {
      window.setCurrentFont(font);
      //Utils.userMessage("Font", font.toString(), 2);
	    setVisible(false);
    }
		else if(source == cancel)
			setVisible(false);

		else if(source == chooseSize)
		{
			fontSize = Integer.parseInt((String)chooseSize.getSelectedItem());
			font = font.deriveFont((float)fontSize);
			fontDisplay.setFont(font);
			fontDisplay.repaint();
		}
	}

    // List selection listener method
	public void valueChanged(ListSelectionEvent e)
	{
		if(!e.getValueIsAdjusting())
		{
			font = new Font((String)fontList.getSelectedValue(), fontStyle, fontSize);
			fontDisplay.setFont(font);
			fontDisplay.repaint();
		}
	}

  class StyleListener implements ItemListener
  {
    public StyleListener(int style)
    {
      this.style = style;
    }

    public void itemStateChanged(ItemEvent e)
    {
      if(e.getStateChange()==ItemEvent.SELECTED)
        fontStyle |= style;
      else
        fontStyle &= ~style;

      font = font.deriveFont(fontStyle);
      fontDisplay.setFont(font);
      fontDisplay.repaint();
    }

    private int style;
  }

  private jpe window;
  private Utils Utils = new Utils(window);
  private Font font;
  private int fontStyle;
  private int fontSize;
	private String fontName;
  private JButton ok;
  private JButton cancel;
  private JList fontList;
  private JComboBox chooseSize;
  private JLabel fontDisplay;

}

