C
C
      SUBROUTINE GGHIOF(NEVENT,EEN,NA,NZ)
C**********************************************************************
C
C     interface to call PHOJET (variable energy run) for
C     gamma-gamma collisions via heavy ions (form factor approach)
C
C     input:     EEN     LAB system energy per nucleon
C                NA      atomic number of ion/hadron
C                NZ      charge number of ion/hadron
C                NEVENT  number of events to generate
C            from COMMON /LEPCUT/:
C                YMIN1,2 lower limit of Y
C                        (energy fraction taken by photon from hadron)
C                YMAX1,2 upper cutoff for Y, necessary to avoid
C                        underflows
C                Q2MIN1,2 minimum Q**2 of photons (should be set to 0)
C                Q2MAX1,2 maximum Q**2 of photons (if necessary,
C                        corrected according size of hadron)
C
C      currently implemented approximation similar to:
C                E.Papageorgiu PhysLettB250(1990)155
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( ZERO = 0.D0,
     &            ONE  = 1.D0,
     &            TWO  = 2.D0,
     &            PI   = 3.14159265359D0 )
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
C  lepton/hadron-photon kinematics
      COMMON /PHOSRC/ PINI(5,2),PFIN(5,2),PGAM(5,2),
     &                GYY(2),GQ2(2),GGECM,GAIMP(2),PFTHE(2),PFPHI(2),
     &                IDPSRC(2),IDBSRC(2),RADSRC(2),AMSRC(2),GAMSRC(2)
      COMMON /LEPCUT/ ECMIN,ECMAX,EEMIN1,EEMIN2,
     &                YMIN1,YMAX1,YMIN2,YMAX2,
     &                Q2MIN1,Q2MAX1,Q2MIN2,Q2MAX2,
     &                THMIN1,THMAX1,THMIN2,THMAX2
C
      DIMENSION P1(4),P2(4),BIMP(2,2)
      CHARACTER*72 TITLE
C
      WRITE(6,'(2(/1X,A))') 'GGHIOF:GAMMA-GAMMA EVENT GENERATION',
     &                      '==================================='
C  hadron size and mass
      FM2GEV = 5.07D0
      HIMASS = DBLE(NA)*0.938D0
      HIMA2  = HIMASS**2
      HIRADI = 1.2D0*FM2GEV*DBLE(NA)**0.333
      ALPHA  = DBLE(NZ**2)/137.D0
C  correct Q2MAX1,2 according to hadron size
      Q2MAXH = 2.D0/HIRADI**2
      Q2MAX1 = MIN(Q2MAX1,Q2MAXH)
      Q2MAX2 = MIN(Q2MAX2,Q2MAXH)
      IF(Q2MAX1.LT.1.D-20) Q2MAX1 = Q2MAXH
      IF(Q2MAX2.LT.1.D-20) Q2MAX2 = Q2MAXH
C  total hadron / heavy ion energy
      EE = EEN*DBLE(NA)
      GAMMA = EE/HIMASS
C  setup /PHOSRC/
      GAMSRC(1) = GAMMA
      GAMSRC(2) = GAMMA
      RADSRC(1) = HIRADI
      RADSRC(2) = HIRADI
      AMSRC(1)  = HIMASS
      AMSRC(1)  = HIMASS
C  kinematic limitations
      YMI = (ECMIN/(2.D0*EE))**2
      IF(YMIN1.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)') 
     &    'GGHIOF:DEBUG:YMIN1 INCREASED TO (OLD/NEW)',YMIN1,YMI
        YMIN1 = YMI
      ELSE IF(YMIN1.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGHIOF:WARNING:','ECM-CUT CORRESPONDS TO YMIN1 OF',YMI,
     &    '  INSTEAD OF',YMIN1
      ENDIF
      IF(YMIN2.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)') 
     &    'GGHIOF:DEBUG:YMIN2 INCREASED TO (OLD/NEW)',YMIN2,YMI
        YMIN2 = YMI
      ELSE IF(YMIN2.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGHIOF:WARNING:','ECM-CUT CORRESPONDS TO YMIN2 OF',YMI,
     &    '  INSTEAD OF',YMIN2
      ENDIF
C  kinematic limitation
      Q2LOW1 = MAX(Q2MIN1,HIMA2*YMIN1**2/(ONE-YMIN1))
      Q2LOW2 = MAX(Q2MIN2,HIMA2*YMIN2**2/(ONE-YMIN2))
C  debug output
      WRITE(6,'(/6X,A,2I4)')   'MASS NUMBER, CHARGE NUMBER  ',NA,NZ
      WRITE(6,'(6X,A,E12.5)') 'HADRON MASS (GeV)           ',HIMASS
      WRITE(6,'(6X,A,E12.5)') 'HADRON RADIUS (GeV**-1)     ',HIRADI
      WRITE(6,'(6X,A,2E12.5)') 'Q**2 RANGE PHOTON 1 (GEV**2)',Q2LOW1,
     &  Q2MAX1
      WRITE(6,'(6X,A,2E12.5)') 'Q**2 RANGE PHOTON 2 (GEV**2)',Q2LOW2,
     &  Q2MAX2
      WRITE(6,'(6X,A,2E12.5)') 'Y RANGE PHOTON 1            ',YMIN1,
     &  YMAX1
      WRITE(6,'(6X,A,2E12.5)') 'Y RANGE PHOTON 2            ',YMIN2,
     &  YMAX2
      WRITE(6,'(6X,A,2E12.5)') 'SQRT(S) PER NUCLEON, TOTAL  ',
     &  2.D0*EEN,2.D0*EE
      WRITE(6,'(6X,A,I10)')   'EVENTS TO PROCESS           ',NEVENT
      IF(Q2LOW1.GE.Q2MAX1) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOF:ERROR:INCONSISTENT Q**2 RANGE 1',Q2LOW1,Q2MAX1
        CALL POABRT
      ENDIF
      IF(Q2LOW2.GE.Q2MAX2) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOF:ERROR:INCONSISTENT Q**2 RANGE 2',Q2LOW2,Q2MAX2
        CALL POABRT
      ENDIF
C  hadron numbers set to 0
      IDPSRC(1) = 0
      IDPSRC(2) = 0
      IDBSRC(1) = 0
      IDBSRC(2) = 0
C
      MAXTAB = 100
      YMAX = YMAX1
      YMIN = YMIN1
      XMAX = LOG(YMAX)
      XMIN = LOG(YMIN)
      XDEL = XMAX-XMIN
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      DO 100 I=1,MAXTAB
        Y1 = EXP(XMIN+DELLY*DBLE(I-1))
        Q2LOW1 = MAX(Q2MIN1,HIMA2*Y1*Y1/(ONE-Y1))
        IF(Q2LOW1.GE.Q2MAX1) THEN
          WRITE(6,'(/1X,A,2E12.4)') 
     &      'GGHIOF:DEBUG:YMAX1 CHANGED FROM/TO',YMAX1,Y1
          YMAX1 = MIN(Y1,YMAX1)
          GOTO 101
        ENDIF
 100  CONTINUE
 101  CONTINUE
      YMAX = YMAX2
      YMIN = YMIN2
      XMAX = LOG(YMAX)
      XMIN = LOG(YMIN)
      XDEL = XMAX-XMIN
      DELLY = LOG(YMAX/YMIN)/DBLE(MAXTAB-1)
      DO 102 I=1,MAXTAB
        Y1 = EXP(XMIN+DELLY*DBLE(I-1))
        Q2LOW2 = MAX(Q2MIN2,HIMA2*Y1*Y1/(ONE-Y1))
        IF(Q2LOW2.GE.Q2MAX2) THEN
          WRITE(6,'(/1X,A,2E12.4)') 
     &      'GGHIOF:DEBUG:YMAX2 CHANGED FROM/TO',YMAX2,Y1
          YMAX2 = MIN(Y1,YMAX2)
          GOTO 103
        ENDIF
 102  CONTINUE
 103  CONTINUE
      YMI = (ECMIN/(2.D0*EE))**2/YMAX2
      IF(YMI.GT.YMIN1) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOF:DEBUG:YMIN1 CHANGED FROM/TO',YMIN1,YMI
        YMIN1 = YMI
      ENDIF
      YMI = (ECMIN/(2.D0*EE))**2/YMAX1
      IF(YMI.GT.YMIN2) THEN
        WRITE(6,'(/1X,A,2E12.4)')
     &    'GGHIOF:DEBUG:YMIN2 CHANGED FROM/TO',YMIN2,YMI
        YMIN2 = YMI
      ENDIF
C
      X1MAX = LOG(YMAX1)
      X1MIN = LOG(YMIN1)
      X1DEL = X1MAX-X1MIN
      X2MAX = LOG(YMAX2)
      X2MIN = LOG(YMIN2)
      X2DEL = X2MAX-X2MIN
      DELLY = LOG(YMAX1/YMIN1)/DBLE(MAXTAB-1)
      FLUX = 0.D0
*     WRITE(6,'(/1X,A,I5)')
*    &  'GGHIOF:DEBUG:TABLE OF RAW PHOTON FLUX (SIDE 1)',MAXTAB
      DO 105 I=1,MAXTAB
        Y1 = EXP(X1MIN+DELLY*DBLE(I-1))
        Q2LOW1 = MAX(Q2MIN1,HIMA2*Y1*Y1/(ONE-Y1))
        FF = ((ONE+(ONE-Y1)**2)/Y1*LOG(Q2MAX1/Q2LOW1)
     &        -TWO*HIMA2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1))*ALPHA/(TWO*PI)
        FLUX = FLUX+Y1*FF
*       WRITE(6,'(5X,2E15.4)') Y1,FF
 105  CONTINUE
      FLUX = FLUX*DELLY
      WRITE(6,'(1X,A,E12.4)') 
     &  'GGHIOF:DEBUG:INTEGRATED FLUX (ONE SIDE):',FLUX
C
      Q2LOW1 = MAX(Q2MIN1,HIMA2*YMIN1**2/(ONE-YMIN1))
      Q2LOW2 = MAX(Q2MIN2,HIMA2*YMIN2**2/(ONE-YMIN2))
      Y1 = YMIN1
      Y2 = YMIN2
      WGMAX = ((ONE+(ONE-Y1)**2)*LOG(Q2MAX1/Q2LOW1)
     &         -TWO*HIMA2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1)*Y1)
     &       *((ONE+(ONE-Y2)**2)*LOG(Q2MAX2/Q2LOW2)
     &         -TWO*HIMA2*Y2*(ONE/Q2LOW2-ONE/Q2MAX2)*Y2)
C
C  photon 1
      EGAM = YMAX1*EE
      P1(1) = ZERO
      P1(2) = ZERO
      P1(3) = EGAM
      P1(4) = EGAM
C  photon 2
      EGAM = YMAX2*EE
      P2(1) = ZERO
      P2(2) = ZERO
      P2(3) = -EGAM
      P2(4) = EGAM
      CALL SETPAR(1,22,0,ZERO)
      CALL SETPAR(2,22,0,ZERO)
      CALL EVENT(-1,P1,P2,SIGMAX,IREJ)
      CALL POHIST(-1,WEIGHT)
      CALL POLUHI(-1,WEIGHT)
C
C  generation of events, flux calculation
      ECUP = 2.D0*SQRT(YMAX1*YMAX2*EE*EE)*1.1D0
      YMINL = LOG(ECMIN)
      YMAXL = LOG(ECUP)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IGGLUM)
      YMINL = LOG(YMIN1)
      YMAXL = LOG(YMAX1)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLX1)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLY1)
      YMINL = LOG(YMIN2)
      YMAXL = LOG(YMAX2)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLX2)
      CALL NEWHIS(YMINL,YMAXL,ZERO,P1,49,IHFLY2)
      AY1  = ZERO
      AY2  = ZERO
      AYS1 = ZERO
      AYS2 = ZERO
      Q21MIN = 1.D30
      Q22MIN = 1.D30
      Q21MAX = 0.D0
      Q22MAX = 0.D0
      Q21AVE = 0.D0
      Q22AVE = 0.D0
      Q21AV2 = 0.D0
      Q22AV2 = 0.D0
      YY1MIN = 1.D30
      YY2MIN = 1.D30
      YY1MAX = 0.D0
      YY2MAX = 0.D0
      NITER = NEVENT
      ITRY = 0
      ITRW = 0
      DO 200 I=1,NITER
C  sample y1, y2
 150    CONTINUE
        ITRY = ITRY+1
 175    CONTINUE
          ITRW = ITRW+1
          Y1 = EXP(X1DEL*RNDM(AY1)+X1MIN)
          Y2 = EXP(X2DEL*RNDM(AY2)+X2MIN)
          Q2LOW1 = MAX(Q2MIN1,HIMA2*Y1**2/(ONE-Y1))
          IF(Q2LOW1.GE.Q2MAX1) GOTO 175
          Q2LOW2 = MAX(Q2MIN2,HIMA2*Y2**2/(ONE-Y2))
          IF(Q2LOW2.GE.Q2MAX2) GOTO 175
          Q2LOG1 = LOG(Q2MAX1/Q2LOW1)
          Q2LOG2 = LOG(Q2MAX2/Q2LOW2)
          WGH = ((ONE+(ONE-Y1)**2)*Q2LOG1
     &        -TWO*HIMA2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1)*Y1)
     &         *((ONE+(ONE-Y2)**2)*Q2LOG2
     &        -TWO*HIMA2*Y2*(ONE/Q2LOW2-ONE/Q2MAX2)*Y2)
          IF(WGMAX.LT.WGH) THEN
            WRITE(6,'(1X,A,4E12.5)')
     &        'GGHIOF:WEIGHT ERROR:',Y1,Y2,WGMAX,WGH
          ENDIF
        IF(RNDM(AYS1)*WGMAX.GT.WGH) GOTO 175
C  sample Q2
        YEFF = ONE+(ONE-Y1)**2
 185    CONTINUE
          Q2P1 = Q2LOW1*EXP(Q2LOG1*RNDM(Y1))
          WEIGHT = (YEFF-2.D0*(ONE-Y1)*Q2LOW1/Q2P1)/YEFF
        IF(WEIGHT.LT.RNDM(Q2P1)) GOTO 185
        YEFF = ONE+(ONE-Y2)**2
 186    CONTINUE
          Q2P2 = Q2LOW2*EXP(Q2LOG2*RNDM(Y2))
          WEIGHT = (YEFF-2.D0*(ONE-Y2)*Q2LOW2/Q2P2)/YEFF
        IF(WEIGHT.LT.RNDM(Q2P2)) GOTO 186
C  impact parameter
        GAIMP(1) = 1.D0/SQRT(Q2P1)
        GAIMP(2) = 1.D0/SQRT(Q2P2)
C  form factor (squared)
        FF21 = 1.D0
        IF(GAIMP(1).LT.HIRADI) FF21 = 0.D0
        FF22 = 1.D0
        IF(GAIMP(2).LT.HIRADI) FF22 = 0.D0
        IF(RNDM(Q2P1).GE.FF21*FF22) GOTO 175
C  do the hadrons overlap?
        IF(ISWMDL(26).GT.0) THEN
          DO 190 K=1,2
            CALL SFECFE(SIF,COF)
            BIMP(1,K) = SIF*GAIMP(K)
            BIMP(2,K) = COF*GAIMP(K)
 190      CONTINUE
          BBABS = SQRT((BIMP(1,1)-BIMP(1,2))**2
     &                 +(BIMP(2,1)-BIMP(2,2))**2)
          IF(BBABS.LT.HIRADI+HIRADI) GOTO 175
        ENDIF
C  photon data
        GYY(1) = Y1
        GQ2(1) = Q2P1
        GYY(2) = Y2
        GQ2(2) = Q2P2
C
        Y1L = LOG(Y1)
        Y2L = LOG(Y2)
        CALL FILHIS(Y1L,ONE,IHFLX1)
        CALL ADDHIS(IHFLX1)
        CALL FILHIS(Y2L,ONE,IHFLX2)
        CALL ADDHIS(IHFLX2)
C  incoming hadron 1
        PINI(1,1) = ZERO
        PINI(2,1) = ZERO
        PINI(3,1) = EE
        PINI(4,1) = EE
        PINI(5,1) = ZERO
C  outgoing hadron 1
        YQ2 = SQRT((ONE-Y1)*Q2P1)
        Q2E = Q2P1/(4.D0*EE)
        E1Y = EE*(ONE-Y1)
        CALL SFECFE(SIF,COF)
        PFIN(1,1) = YQ2*COF
        PFIN(2,1) = YQ2*SIF
        PFIN(3,1) = E1Y-Q2E
        PFIN(4,1) = E1Y+Q2E
        PFIN(5,1) = ZERO
        PFPHI(1) = ATAN2(COF,SIF)
        PFTHE(1) = ACOS((E1Y-Q2E)/(Q2E+E1Y))
C  photon 1
        P1(1) = -PFIN(1,1)
        P1(2) = -PFIN(2,1)
        P1(3) = PINI(3,1)-PFIN(3,1)
        P1(4) = PINI(4,1)-PFIN(4,1)
C  incoming hadron 2
        PINI(1,2) = ZERO
        PINI(2,2) = ZERO
        PINI(3,2) = -EE
        PINI(4,2) = EE
        PINI(5,2) = ZERO
C  outgoing hadron 2
        YQ2 = SQRT((ONE-Y2)*Q2P2)
        Q2E = Q2P2/(4.D0*EE)
        E1Y = EE*(ONE-Y2)
        CALL SFECFE(SIF,COF)
        PFIN(1,2) = YQ2*COF
        PFIN(2,2) = YQ2*SIF
        PFIN(3,2) = -E1Y+Q2E
        PFIN(4,2) = E1Y+Q2E
        PFIN(5,2) = ZERO
        PFPHI(2) = ATAN2(COF,SIF)
        PFTHE(2) = ACOS((Q2E-E1Y)/(Q2E+E1Y))
C  photon 2
        P2(1) = -PFIN(1,2)
        P2(2) = -PFIN(2,2)
        P2(3) = PINI(3,2)-PFIN(3,2)
        P2(4) = PINI(4,2)-PFIN(4,2)
C  ECMS cut
        GGECM = (P1(4)+P2(4))**2-(P1(1)+P2(1))**2
     &         -(P1(2)+P2(2))**2-(P1(3)+P2(3))**2
        IF(GGECM.LT.0.1D0) GOTO 175
        GGECM = SQRT(GGECM)
        IF((GGECM.LT.ECMIN).OR.(GGECM.GT.ECMAX)) GOTO 175
C
        PGAM(1,1) = P1(1)
        PGAM(2,1) = P1(2)
        PGAM(3,1) = P1(3)
        PGAM(4,1) = P1(4)
        PGAM(5,1) = -SQRT(Q2P1)
        PGAM(1,2) = P2(1)
        PGAM(2,2) = P2(2)
        PGAM(3,2) = P2(3)
        PGAM(4,2) = P2(4)
        PGAM(5,2) = -SQRT(Q2P2)
        CALL PRESEL(5,IREJ)
        IF(IREJ.NE.0) GOTO 175
C  event generation
        CALL EVENT(1,P1,P2,SIGCUR,IREJ)
        IF(IREJ.NE.0) GOTO 150
        GGECML = LOG(GGECM)
        CALL FILHIS(GGECML,ONE,IGGLUM)
        CALL ADDHIS(IGGLUM)
        CALL FILHIS(Y1L,ONE,IHFLY1)
        CALL ADDHIS(IHFLY1)
        CALL FILHIS(Y2L,ONE,IHFLY2)
        CALL ADDHIS(IHFLY2)
C  statistics
        AY1  = AY1+Y1
        AYS1 = AYS1+Y1*Y1
        AY2  = AY2+Y2
        AYS2 = AYS2+Y2*Y2
        Q21MIN = MIN(Q21MIN,Q2P1)
        Q22MIN = MIN(Q22MIN,Q2P2)
        Q21MAX = MAX(Q21MAX,Q2P1)
        Q22MAX = MAX(Q22MAX,Q2P2)
        YY1MIN = MIN(YY1MIN,Y1)
        YY2MIN = MIN(YY2MIN,Y2)
        YY1MAX = MAX(YY1MAX,Y1)
        YY2MAX = MAX(YY2MAX,Y2)
        Q21AVE = Q21AVE+Q2P1
        Q22AVE = Q22AVE+Q2P2
        Q21AV2 = Q21AV2+Q2P1*Q2P1
        Q22AV2 = Q22AV2+Q2P2*Q2P2
C  histograms
        CALL POHIST(1,ONE)
        CALL POLUHI(1,ONE)
 200  CONTINUE
C
      WGY  = WGMAX*DBLE(ITRY)/DBLE(ITRW)*(ALPHA/(TWO*PI))**2
      WGY  = WGY*LOG(YMAX1/YMIN1)*LOG(YMAX2/YMIN2)
      AY1  = AY1/DBLE(NITER)
      AYS1 = AYS1/DBLE(NITER)
      DAY1 = SQRT((AYS1-AY1**2)/DBLE(NITER))
      AY2  = AY2/DBLE(NITER)
      AYS2 = AYS2/DBLE(NITER)
      DAY2 = SQRT((AYS2-AY2**2)/DBLE(NITER))
      Q21AVE = Q21AVE/DBLE(NITER)
      Q21AV2 = Q21AV2/DBLE(NITER)
      Q21AV2 = SQRT((Q21AV2-Q21AVE**2)/DBLE(NITER))
      Q22AVE = Q22AVE/DBLE(NITER)
      Q22AV2 = Q22AV2/DBLE(NITER)
      Q22AV2 = SQRT((Q22AV2-Q22AVE**2)/DBLE(NITER))
      WEIGHT = WGY*SIGMAX*DBLE(NITER)/DBLE(ITRY)
C  output of statistics, histograms
      WRITE(6,'(//1X,A,/1X,A,1PE12.3,A,/1X,A)')
     &'=========================================================',
     &' *****   simulated cross section: ',WEIGHT,' mb  *****',
     &'========================================================='
      WRITE(6,'(//1X,A,3I10)')
     &  'GGHIOF:SUMMARY:NITER,ITRY,ITRW',NITER,ITRY,ITRW
      WRITE(6,'(1X,A,1P2E12.4)') 'EFFECTIVE WEIGHT (FLUX,TOTAL)',
     &  WGY,WEIGHT
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y1,DY1               ',
     &  AY1,DAY1
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y2,DY2               ',
     &  AY2,DAY2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 1     ',
     &  YY1MIN,YY1MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 2     ',
     &  YY2MIN,YY2MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Q2,DQ2 PHOTON 1      ',
     &  Q21AVE,Q21AV2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Q2 RANGE PHOTON 1    ',
     &  Q21MIN,Q21MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Q2,DQ2  PHOTON 2     ',
     &  Q22AVE,Q22AV2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Q2 RANGE PHOTON 2    ',
     &  Q22MIN,Q22MAX
C
      CALL EVENT(-2,P1,P2,WEIGHT,IREJ)
      IF(NITER.GT.1) THEN
        CALL POHIST(-2,WEIGHT)
        CALL POLUHI(-2,WEIGHT)
      ELSE
        WRITE(6,'(1X,A,I4)') 'GGHIOF:NO OUTPUT OF HISTOGRAMS',NITER
      ENDIF
C
      TITLE = ' HEAVY ION PHOTON-PHOTON LUMINOSITY DPHI/D(ECM-GG)'
      CALL OUTHIS(IGGLUM,3,TITLE,WGY,1)
      TITLE = ' HEAVY ION PHOTON-PHOTON CROSS SECTION DSIG/D(ECM-GG)'
      CALL OUTHIS(IGGLUM,3,TITLE,WEIGHT,1)
      FAC = SQRT(WGY)
      TITLE = ' HEAVY ION PHOTON FLUX (FORM FACTOR) SIDE 1'
      CALL OUTHIS(IHFLX1,3,TITLE,FAC,1)
      TITLE = ' HEAVY ION PHOTON FLUX (FORM FACTOR) SIDE 2'
      CALL OUTHIS(IHFLX2,3,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 1 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY1,3,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 2 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY2,3,TITLE,FAC,1)
      END
