C
C
      SUBROUTINE PODBLE(IMOTH1,IMOTH2,MSOFT,MHARD,IMODE,IREJ)
C**********************************************************************
C
C     preparation of common HEPEVS for double-pomeron scattering
C
C     input:   IMOTH1/2   index of mother particles in COMMON HEPEVS
C
C              IMODE   1  sampling of pomeron-pomeron scattering
C                     -1  initialization
C                     -2  output of statistics
C
C     output:   MSOFT     number of generated soft chains
C               MHARD     number of generated hard chains
C               IREJ      0  accepted
C                         1  rejected
C                        50  user rejection
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER(NTAB = 20,
     &          EPS  = 1.D-10,
     &          IONE = 1,
     &          ONE  = 1.D0,
     &          PIMASS = 0.13D0,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 1.D-10,
     &          ZERO = 0.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
      PARAMETER (NMXJ=40)
      CHARACTER*10 REJTIT
      COMMON /REJSTA/ IFAIL(NMXJ),REJTIT(NMXJ)
C
      COMMON /PROCES/ IPROCE,IDNODF,IDIFR1,IDIFR2,IDDPOM,IPRON(15)
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER (IEETAB=20)
      COMMON /XSETAB/ SIGTAB(4,70,IEETAB),SIGECM(4,IEETAB),ISIMAX
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      DIMENSION PD(4)
C
      IREJ = 0
      IP = 4
C  select first diffraction
      IF(RNDM(XI).GT.0.5D0) THEN
        IPAR1 = 1
        IPAR2 = 0
      ELSE
        IPAR1 = 0
        IPAR2 = 1
      ENDIF
      NHEPS = NHEP
      ITRY2 = 0
      ITRYM = 1000
C  generate diffractive pomeron scatterings
 60   CONTINUE
      ITRY2 = ITRY2+1
      IF(ITRY2.GT.1) THEN
        IFAIL(39) = IFAIL(39)+1
        IF(ITRY2.GE.ITRYM) GOTO 50
      ENDIF
      I1 = IPAR1
      I2 = IPAR2
      JDAHEP(1,IMOTH1) = 0
      JDAHEP(2,IMOTH1) = 0
      JDAHEP(1,IMOTH2) = 0
      JDAHEP(2,IMOTH2) = 0
      ISTHEP(IMOTH1) = 1
      ISTHEP(IMOTH2) = 1
      NHEP = NHEPS
      CALL PODIFF(I1,I2,IMOTH1,IMOTH2,MSOFT,MHARD,2,IREJ)
      IF(IREJ.NE.0) GOTO 50
      CALL PODIFF(I2,I1,IMOTH1,IMOTH2,MSOFT,MHARD,2,IREJ)
      IF(IREJ.NE.0) GOTO 50
      DO 100 I2 = NHEP,1,-1
        IF(IDHEP(I2).EQ.45) GOTO 110
 100  CONTINUE
 110  CONTINUE
      DO 120 I1 = I2-1,1,-1
        IF(IDHEP(I1).EQ.45) GOTO 130
 120  CONTINUE
 130  CONTINUE
C  mass of pomeron-pomeron system
      DO 140 I=1,4
        PD(I) = PHEP(I,I1)+PHEP(I,I2)
 140  CONTINUE
      XMASS = PD(4)**2-PD(1)**2-PD(2)**2-PD(3)**2
      IF(IDEB(59).GE.20) WRITE(6,'(1X,A,2I3,E12.4)')
     &  'PODBLE:DEBUG:IPOM1,IPOM2,MASS**2',I1,I2,XMASS
      IF(XMASS.LT.0.1D0) GOTO 60
      XMASS = SQRT(XMASS)
      IF(XMASS.LT.PARMDL(71)) GOTO 60
C  get cross sections
      IF(XMASS.LE.SIGECM(IP,1)) THEN
        IN1 = 1
        IN2 = 1
      ELSE IF(XMASS.LT.SIGECM(IP,ISIMAX)) THEN
        DO 55 I=2,ISIMAX
          IF(XMASS.LE.SIGECM(IP,I)) GOTO 210
 55     CONTINUE
 210    CONTINUE
        IN1 = I-1
        IN2 = I
      ELSE
        WRITE(6,'(/1X,A,2E12.3)')
     &    'PODBLE:WARNING:TOO HIGH XM',XMASS,SIGECM(IP,ISIMAX)
        IN1 = ISIMAX
        IN2 = ISIMAX
      ENDIF
      FAC2=0.D0
      IF(IN1.NE.IN2) FAC2=LOG(XMASS/SIGECM(IP,IN1))
     &                    /LOG(SIGECM(IP,IN2)/SIGECM(IP,IN1))
      FAC1=1.D0-FAC2
      STOT = FAC2*SIGTAB(IP,1,IN2)+FAC1*SIGTAB(IP,1,IN1)
      SELA = FAC2*SIGTAB(IP,2,IN2)+FAC1*SIGTAB(IP,2,IN1)
      SINE = STOT-SELA
      SDIR = FAC2*SIGTAB(IP,29,IN2)+FAC1*SIGTAB(IP,29,IN1)
      SDIR = MAX(ZERO,SDIR)/SINE
C  sample interaction
      IF((RNDM(FAC1).LT.SDIR).AND.(IPRON(11).NE.0)) THEN
        IDIR = 1
        ISAM = 0
        JSAM = 0
        KSAM = 0
        LSAM = 0
        MSAM = 0
      ELSE
        CALL SAMPRB(XMASS,IP,ISAM,JSAM,KSAM,LSAM,MSAM)
      ENDIF
C  suppress hard scattering
      IF(IPRON(11).EQ.0) KSAM = 0
C  suppress multiple interaction
      IF(ISWMDL(25).EQ.0) THEN
        KSAM = 0
        IDIR = 0
        IF(ISAM+KSAM.GT.0) THEN
          ISAM = 1
          JSAM = 0
        ELSE
          JSAM = 1
        ENDIF
      ELSE IF(ISWMDL(25).EQ.1) THEN
        IF(IDIR.GT.0) THEN
        ELSE IF(KSAM.GT.0) THEN
          KSAM = 1
          ISAM = 0
          JSAM = 0
        ELSE IF(ISAM.GT.0) THEN
          ISAM = 1
          JSAM = 0
        ELSE
          JSAM = 1
        ENDIF
      ELSE IF(ISWMDL(25).EQ.2) THEN
        KSAM = MIN(KSAM,1)
      ELSE IF(ISWMDL(25).EQ.3) THEN
        IF(ISAM.GT.0) THEN
          ISAM = 1
          JSAM = 0
        ELSE
          JSAM = 1
        ENDIF
      ENDIF
C  rejection loop
 200  CONTINUE
      IF(ISAM+JSAM+KSAM+IDIR.EQ.0) JSAM = 1
C  debug output
      IF(IDEB(59).GE.15) WRITE(6,'(1X,A,/5X,I3,E12.4,4I5)')
     &  'PODBLE:DEBUG:IP,XMASS,ISAM,JSAM,KSAM,IDIR,',
     &  IP,XMASS,ISAM,JSAM,KSAM,IDIR
C  pomeron-pomeron scattering
      IF(IDIR.GT.0) THEN
        IPAR = 4
      ELSE IF(KSAM.GT.0) THEN
        IPAR = 3
      ELSE IF(ISAM.GT.0) THEN
        IPAR = 2
      ELSE
        IPAR = 1
      ENDIF
      IDDPOM = IPAR
C  debug information
      IF(ISAM+JSAM.GT.0) KSDPO = 1
      IF(KSAM+IDIR.GT.0) KHDPO = 1
      KSPOM = ISAM
      KSREG = JSAM
      KHPOM = KSAM
      KHDIR = IDIR
      KSTRG = 0
      KSLOO = 0
      CALL STDPAR(I1,I2,ISAM,JSAM,KSAM,IDIR,IREJ)
      IF(IREJ.NE.0) THEN
        IFAIL(21) = IFAIL(21)+1
        IF(IPAR.GT.1) THEN
          IF(IPAR.EQ.3) IFAIL(9) = IFAIL(9)+1
          IF(IDIR.GT.0) THEN
            IFAIL(10) = IFAIL(10)+1
            IDIR = 0
          ELSE IF(KSAM.GT.0) THEN
            KSAM = KSAM-1
          ELSE IF(ISAM.GT.0) THEN
            ISAM = ISAM-1
          ENDIF
          GOTO 200
        ELSE
          IF(IDEB(59).GE.2) WRITE(6,'(1X,A,2I3,E11.3)')
     &      'PODBLE:WARNING:REJECTION STDPAR (I,IPAR,XM)',
     &      I,IPAR,XMASS
          GOTO 50
        ENDIF
      ENDIF
C  comment line for central diffraction
      CALL REGPAR(40,4,IPAR,IMOTH1,IMOTH2,PD(1),PD(2),PD(3),PD(4),
     &            I1,I2,IDHEP(IMOTH1),IDHEP(IMOTH2),IPOS,1)
      PHEP(5,IPOS) = XMASS
C  debug output
      IF(IDEB(59).GE.15) THEN
        WRITE(6,'(2(/1X,A))') 'PODBLE:DEBUG OUTPUT:',
     &    '===================='
        CALL POPREV(0)
      ENDIF
      RETURN
C  rejection
 50   CONTINUE
      IREJ = 1
      IFAIL(40) = IFAIL(40)+1
      IF(IDEB(59).GE.3) THEN
        WRITE(6,'(1X,A)')
     &    'PODBLE:WARNING:REJECTION (ITRY,ITRYM)',ITRY2,ITRYM
        IF(IDEB(59).GE.10) THEN
          CALL POPREV(0)
        ELSE
          CALL POPREV(-1)
        ENDIF
      ENDIF
      END
